@@ -30,9 +30,7 @@
 #include "ptyxis-application.h"
 #include "ptyxis-build-ident.h"
 #include "ptyxis-client.h"
-#include "ptyxis-container-menu.h"
 #include "ptyxis-preferences-window.h"
-#include "ptyxis-profile-menu.h"
 #include "ptyxis-session.h"
 #include "ptyxis-settings.h"
 #include "ptyxis-util.h"
@@ -48,8 +46,6 @@ struct _PtyxisApplication
   GListStore          *profiles;
   PtyxisSettings      *settings;
   PtyxisShortcuts     *shortcuts;
-  PtyxisContainerMenu *container_menu;
-  PtyxisProfileMenu   *profile_menu;
   char                *next_title_prefix;
   char                *system_font_name;
   GDBusProxy          *portal;
@@ -61,6 +57,7 @@ struct _PtyxisApplication
   guint                overlay_scrollbars : 1;
   guint                client_is_fallback : 1;
   guint                maximize : 1;
+  guint                fullscreen : 1;
 };
 
 static void ptyxis_application_about             (GSimpleAction *action,
@@ -105,9 +102,11 @@ G_DEFINE_FINAL_TYPE (PtyxisApplication, ptyxis_application, ADW_TYPE_APPLICATION
 
 enum {
   PROP_0,
+  PROP_CONTAINERS,
   PROP_DEFAULT_PROFILE,
   PROP_OS_NAME,
   PROP_OVERLAY_SCROLLBARS,
+  PROP_PROFILES,
   PROP_SYSTEM_FONT_NAME,
   N_PROPS
 };
@@ -139,18 +138,24 @@ should_ignore_osc_title (PtyxisApplication *self,
 }
 
 static void
-maybe_maximize (PtyxisApplication *self,
-                PtyxisWindow      *window)
+maybe_maximize_or_fullscreen (PtyxisApplication *self,
+                              PtyxisWindow      *window)
 {
   g_assert (PTYXIS_IS_APPLICATION (self));
   g_assert (PTYXIS_IS_WINDOW (window));
 
-  if (self->maximize)
+  if (self->maximize || self->fullscreen)
     {
-      /*  unset maximize so it doesn't stick unless it's added again */
-      self->maximize = FALSE;
+      gboolean fullscreen = self->fullscreen;
 
-      gtk_window_maximize (GTK_WINDOW (window));
+      /* unset command-line state */
+      self->maximize = FALSE;
+      self->fullscreen = FALSE;
+
+      if (fullscreen)
+        gtk_window_fullscreen (GTK_WINDOW (window));
+      else
+        gtk_window_maximize (GTK_WINDOW (window));
     }
 }
 
@@ -345,7 +350,7 @@ ptyxis_application_activate (GApplication *app)
       ptyxis_tab_set_ignore_osc_title (tab, should_ignore_osc_title (self, self->next_title_prefix));
       g_clear_pointer (&self->next_title_prefix, g_free);
 
-      maybe_maximize (self, window);
+      maybe_maximize_or_fullscreen (self, window);
 
       gtk_window_present (GTK_WINDOW (window));
     }
@@ -408,6 +413,7 @@ ptyxis_application_command_line (GApplication            *app,
   gboolean new_tab = FALSE;
   gboolean new_window = FALSE;
   gboolean did_restore = FALSE;
+  gboolean fullscreen = FALSE;
   gboolean maximize = FALSE;
   int argc;
 
@@ -431,6 +437,15 @@ ptyxis_application_command_line (GApplication            *app,
   if (g_variant_dict_lookup (dict, "maximize", "b", &maximize))
     self->maximize = !!maximize;
 
+  if (g_variant_dict_lookup (dict, "fullscreen", "b", &fullscreen))
+    self->fullscreen = !!fullscreen;
+
+  if (self->fullscreen && self->maximize)
+    {
+      g_warning ("Both maximize and fullscreen specified, preferring fullscreen");
+      self->maximize = FALSE;
+    }
+
   if (!g_variant_dict_lookup (dict, "tab", "b", &new_tab))
     new_tab = FALSE;
 
@@ -544,7 +559,7 @@ ptyxis_application_command_line (GApplication            *app,
           ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
           ptyxis_window_set_active_tab (window, tab);
 
-          maybe_maximize (self, window);
+          maybe_maximize_or_fullscreen (self, window);
 
           gtk_window_present (GTK_WINDOW (window));
         }
@@ -559,6 +574,8 @@ ptyxis_application_command_line (GApplication            *app,
           ptyxis_tab_set_title_prefix (tab, title);
           ptyxis_tab_set_ignore_osc_title (tab, should_ignore_osc_title (self, title));
 
+          maybe_maximize_or_fullscreen (self, window);
+
           gtk_application_add_window (GTK_APPLICATION (self), GTK_WINDOW (window));
           gtk_window_present (GTK_WINDOW (window));
         }
@@ -640,7 +657,7 @@ ptyxis_application_command_line (GApplication            *app,
       ptyxis_window_add_tab (window, tab);
       ptyxis_window_set_active_tab (window, tab);
 
-      maybe_maximize (self, window);
+      maybe_maximize_or_fullscreen (self, window);
 
       gtk_window_present (GTK_WINDOW (window));
     }
@@ -771,13 +788,6 @@ ptyxis_application_notify_profile_uuids_cb (PtyxisApplication *self,
   g_list_store_splice (self->profiles, 0, n_items, array->pdata, array->len);
 }
 
-static gboolean
-filter_session_container (gpointer item,
-                          gpointer user_data)
-{
-  return g_strcmp0 ("session", ptyxis_ipc_container_get_id (item)) != 0;
-}
-
 static void
 xdg_terminals_list_changed_cb (PtyxisApplication *self,
                                GFile             *file,
@@ -871,8 +881,6 @@ ptyxis_application_startup (GApplication *application)
 {
   static const char *patterns[] = { "org.gnome.*", NULL };
   PtyxisApplication *self = (PtyxisApplication *)application;
-  g_autoptr(GtkFilterListModel) filter_model = NULL;
-  g_autoptr(GtkCustomFilter) filter = NULL;
   g_autoptr(GFile) session_file = get_session_file ();
   g_autoptr(GBytes) session_bytes = NULL;
   g_autoptr(GError) error = NULL;
@@ -946,13 +954,6 @@ ptyxis_application_startup (GApplication *application)
                            self,
                            G_CONNECT_SWAPPED);
 
-  self->profile_menu = ptyxis_profile_menu_new (self->settings);
-
-  filter = gtk_custom_filter_new (filter_session_container, NULL, NULL);
-  filter_model = gtk_filter_list_model_new (g_object_ref (G_LIST_MODEL (self->client)),
-                                            g_object_ref (GTK_FILTER (filter)));
-  self->container_menu = ptyxis_container_menu_new (G_LIST_MODEL (filter_model));
-
   g_action_map_add_action_entries (G_ACTION_MAP (self),
                                    action_entries,
                                    G_N_ELEMENTS (action_entries),
@@ -1035,7 +1036,6 @@ ptyxis_application_shutdown (GApplication *application)
   G_APPLICATION_CLASS (ptyxis_application_parent_class)->shutdown (application);
 
   g_clear_object (&self->xdg_terminals_list_monitor);
-  g_clear_object (&self->profile_menu);
   g_clear_object (&self->profiles);
   g_clear_object (&self->portal);
   g_clear_object (&self->shortcuts);
@@ -1057,6 +1057,10 @@ ptyxis_application_get_property (GObject    *object,
 
   switch (prop_id)
     {
+    case PROP_CONTAINERS:
+      g_value_take_object (value, ptyxis_application_list_containers (self));
+      break;
+
     case PROP_DEFAULT_PROFILE:
       g_value_take_object (value, ptyxis_application_dup_default_profile (self));
       break;
@@ -1069,6 +1073,10 @@ ptyxis_application_get_property (GObject    *object,
       g_value_set_boolean (value, self->overlay_scrollbars);
       break;
 
+    case PROP_PROFILES:
+      g_value_take_object (value, ptyxis_application_list_profiles (self));
+      break;
+
     case PROP_SYSTEM_FONT_NAME:
       g_value_set_string (value, self->system_font_name);
       break;
@@ -1092,6 +1100,12 @@ ptyxis_application_class_init (PtyxisApplicationClass *klass)
   app_class->command_line = ptyxis_application_command_line;
   app_class->open = ptyxis_application_open;
 
+  properties[PROP_CONTAINERS] =
+    g_param_spec_object ("containers", NULL, NULL,
+                         G_TYPE_LIST_MODEL,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+
   properties[PROP_DEFAULT_PROFILE] =
     g_param_spec_object ("default-profile", NULL, NULL,
                          PTYXIS_TYPE_PROFILE,
@@ -1107,7 +1121,13 @@ ptyxis_application_class_init (PtyxisApplicationClass *klass)
   properties [PROP_OVERLAY_SCROLLBARS] =
     g_param_spec_boolean ("overlay-scrollbars", NULL, NULL,
                           FALSE,
-                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+                          (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_PROFILES] =
+    g_param_spec_object ("profiles", NULL, NULL,
+                         G_TYPE_LIST_MODEL,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
 
   properties [PROP_SYSTEM_FONT_NAME] =
     g_param_spec_string ("system-font-name",
@@ -1142,8 +1162,15 @@ ptyxis_application_init (PtyxisApplication *self)
     { "tab", 0, 0, G_OPTION_ARG_NONE, NULL, N_("New terminal tab in active window") },
     { "tab-with-profile", 0, 0, G_OPTION_ARG_STRING, NULL, N_("New terminal tab in active window using the profile UUID"), N_("PROFILE_UUID") },
 
+    /* Standalone (single instance mode) */
+    { "standalone", 's', 0, G_OPTION_ARG_NONE, NULL, N_("Start a new instance, ignoring existing instances") },
+
     { "title", 'T', 0, G_OPTION_ARG_STRING, NULL, N_("Set title for new tab") },
     { "maximize", 0, 0, G_OPTION_ARG_NONE, NULL, N_("Maximize a newly created window") },
+    { "fullscreen", 0, 0, G_OPTION_ARG_NONE, NULL, N_("Fullscreen a newly created window") },
+
+    /* Import a custom .palette file. This works like dragging the file onto preferences */
+    { "import-palette", 0, 0, G_OPTION_ARG_STRING, NULL, N_("Import a Ptyxis palette file"), N_("FILE") },
 
     { NULL }
   };
@@ -1160,7 +1187,11 @@ ptyxis_application_init (PtyxisApplication *self)
   g_string_append_c (summary, '\n');
   g_string_append_printf (summary, "  %s\n", _("Run Custom Command in New Window"));
   g_string_append (summary, "    ptyxis -x \"bash -c 'sleep 3'\"\n");
-  g_string_append (summary, "    ptyxis -- bash -c 'sleep 3'");
+  g_string_append (summary, "    ptyxis -- bash -c 'sleep 3'\n");
+
+  g_string_append_c (summary, '\n');
+  g_string_append_printf (summary, "  %s\n", _("Import a custom palette"));
+  g_string_append (summary, "    ptyxis --import-palette my.palette");
 
   g_application_set_option_context_parameter_string (G_APPLICATION (self), _("[-- COMMAND ARGUMENTS]"));
   g_application_add_main_option_entries (G_APPLICATION (self), main_entries);
@@ -1620,22 +1651,6 @@ ptyxis_application_get_system_font_name (PtyxisApplication *self)
   return self->system_font_name;
 }
 
-GMenuModel *
-ptyxis_application_dup_profile_menu (PtyxisApplication *self)
-{
-  g_return_val_if_fail (PTYXIS_IS_APPLICATION (self), NULL);
-
-  return g_object_ref (G_MENU_MODEL (self->profile_menu));
-}
-
-GMenuModel *
-ptyxis_application_dup_container_menu (PtyxisApplication *self)
-{
-  g_return_val_if_fail (PTYXIS_IS_APPLICATION (self), NULL);
-
-  return g_object_ref (G_MENU_MODEL (self->container_menu));
-}
-
 /**
  * ptyxis_application_list_containers:
  * @self: a #PtyxisApplication
